'use strict';

var protocol = require('@qsocket/protocol');
var zlib = require('zlib');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

var zlib__default = /*#__PURE__*/_interopDefault(zlib);

// src/core/QSocketConnection.ts
var QSocketCompressor = class {
  /**
   * Compresses data using the GZIP algorithm.
   * @param data Data to be compressed.
   * @returns A promise that resolves to the compressed data.
   */
  async toGzip(data) {
    return new Promise((resolve, reject) => {
      zlib__default.default.gzip(data, (error, result) => {
        if (error) {
          reject(new Error(`GZIP compression failed: ${error.message}`));
        } else {
          resolve(result);
        }
      });
    });
  }
  /**
   * Decompresses data that was compressed with the GZIP algorithm.
   * @param data Compressed data.
   * @returns A promise that resolves to the decompressed data.
   */
  async fromGzip(data) {
    return new Promise((resolve, reject) => {
      zlib__default.default.gunzip(data, (error, result) => {
        if (error) {
          reject(new Error(`GZIP decompression failed: ${error.message}`));
        } else {
          resolve(result);
        }
      });
    });
  }
  /**
   * Compresses data using the DEFLATE algorithm.
   * @param data Data to be compressed.
   * @returns A promise that resolves to the compressed data.
   */
  async toDeflate(data) {
    return new Promise((resolve, reject) => {
      zlib__default.default.deflate(data, (error, result) => {
        if (error) {
          reject(new Error(`DEFLATE compression failed: ${error.message}`));
        } else {
          resolve(result);
        }
      });
    });
  }
  /**
   * Decompresses data that was compressed with the DEFLATE algorithm.
   * @param data Compressed data.
   * @returns A promise that resolves to the decompressed data.
   */
  async fromDeflate(data) {
    return new Promise((resolve, reject) => {
      zlib__default.default.inflate(data, (error, result) => {
        if (error) {
          reject(new Error(`DEFLATE decompression failed: ${error.message}`));
        } else {
          resolve(result);
        }
      });
    });
  }
};

// src/core/QSocketHelpers.ts
var contentTypeMap = /* @__PURE__ */ new Map([
  [protocol.EQSocketProtocolContentType.UNDEFINED, "undefined"],
  [protocol.EQSocketProtocolContentType.NULL, "null"],
  [protocol.EQSocketProtocolContentType.BOOLEAN, "boolean"],
  [protocol.EQSocketProtocolContentType.NUMBER, "number"],
  [protocol.EQSocketProtocolContentType.STRING, "string"],
  [protocol.EQSocketProtocolContentType.JSON, "json"],
  [protocol.EQSocketProtocolContentType.BUFFER, "buffer"]
]);
var reverseContentTypeMap = /* @__PURE__ */ new Map([
  ["undefined", protocol.EQSocketProtocolContentType.UNDEFINED],
  ["null", protocol.EQSocketProtocolContentType.NULL],
  ["boolean", protocol.EQSocketProtocolContentType.BOOLEAN],
  ["number", protocol.EQSocketProtocolContentType.NUMBER],
  ["string", protocol.EQSocketProtocolContentType.STRING],
  ["json", protocol.EQSocketProtocolContentType.JSON],
  ["buffer", protocol.EQSocketProtocolContentType.BUFFER]
]);
var contentEncodingMap = /* @__PURE__ */ new Map([
  [protocol.EQSocketProtocolContentEncoding.RAW, "raw"],
  [protocol.EQSocketProtocolContentEncoding.GZIP, "gzip"],
  [protocol.EQSocketProtocolContentEncoding.DEFLATE, "deflate"]
]);
var reverseContentEncodingMap = /* @__PURE__ */ new Map([
  ["raw", protocol.EQSocketProtocolContentEncoding.RAW],
  ["gzip", protocol.EQSocketProtocolContentEncoding.GZIP],
  ["deflate", protocol.EQSocketProtocolContentEncoding.DEFLATE]
]);
function determineContentType(data, contentType) {
  if (contentType) {
    const type = reverseContentTypeMap.get(contentType);
    if (type !== void 0) return type;
  }
  switch (typeof data) {
    case "undefined":
      return protocol.EQSocketProtocolContentType.UNDEFINED;
    case "boolean":
      return protocol.EQSocketProtocolContentType.BOOLEAN;
    case "number":
      return protocol.EQSocketProtocolContentType.NUMBER;
    case "string":
      return protocol.EQSocketProtocolContentType.STRING;
    case "symbol":
      return protocol.EQSocketProtocolContentType.UNDEFINED;
    case "object":
      if (data === null) return protocol.EQSocketProtocolContentType.NULL;
      if (Buffer.isBuffer(data)) return protocol.EQSocketProtocolContentType.BUFFER;
      return protocol.EQSocketProtocolContentType.JSON;
    default:
      return protocol.EQSocketProtocolContentType.UNDEFINED;
  }
}
function determineContentEncoding(contentEncoding) {
  if (contentEncoding) {
    const encoding = reverseContentEncodingMap.get(contentEncoding);
    if (encoding !== void 0) return encoding;
  }
  return protocol.EQSocketProtocolContentEncoding.RAW;
}
function getContentTypeString(contentType) {
  var _a;
  if (contentType === void 0) return "undefined";
  return (_a = contentTypeMap.get(contentType)) != null ? _a : "undefined";
}
function getContentEncodingString(contentEncoding) {
  var _a;
  if (contentEncoding === void 0) return "raw";
  return (_a = contentEncodingMap.get(contentEncoding)) != null ? _a : "raw";
}
function getDefaultProtocolConfig() {
  return {
    compressor: {
      on: true,
      compressor: new QSocketCompressor(),
      compressionFromSize: 1024 * 100
    }
  };
}

// src/core/QSocketConnection.ts
var QSocketConnection = class {
  //#endregion
  //#region Конструктор
  constructor(interaction, namespace) {
    this.events = /* @__PURE__ */ new Map();
    this.closeListeners = [];
    this.interaction = interaction;
    this.namespace = namespace;
  }
  //#endregion
  //#region Методы управления соединением
  /**
   * @description Подписка на событие завершения соединения с клиентом
   */
  onClose(listener) {
    this.closeListeners.push(listener);
  }
  /**
   * @description Закрытие соединения
   */
  $__close() {
    this.closeListeners.forEach((listener) => listener());
  }
  static close(connection) {
    connection.$__close();
  }
  //#endregion
  //#region Методы подписки на события
  /**
   * @description Подписка на событие с поддержкой нескольких слушателей
   */
  on(event, listener, contentType, contentEncoding) {
    this.addEventListener(event, listener, 0 /* ON */, contentType, contentEncoding);
  }
  /**
   * @description Подписка на одноразовое событие
   */
  once(event, listener, contentType, contentEncoding) {
    this.addEventListener(event, listener, 1 /* ONCE */, contentType, contentEncoding);
  }
  //#endregion
  //#region Методы отправки и передачи данных
  /**
   * @description Отправка данных на связанный клиент
   */
  async emit(event, data, contentType, contentEncoding) {
    const message = {
      payload: {
        data,
        "Content-Type": determineContentType(data, contentType),
        "Content-Encoding": determineContentEncoding(contentEncoding)
      },
      meta: {
        type: protocol.EQSocketProtocolMessageType.DATA,
        uuid: this.interaction.uuid.next(),
        namespace: this.namespace.name,
        event
      }
    };
    return await this.interaction.sendData([message]);
  }
  async broadcast(event, data, contentType, contentEncoding) {
    const message = {
      payload: {
        data,
        "Content-Type": determineContentType(data, contentType),
        "Content-Encoding": determineContentEncoding(contentEncoding)
      },
      meta: {
        type: protocol.EQSocketProtocolMessageType.DATA,
        uuid: this.interaction.uuid.next(),
        namespace: this.namespace.name,
        event,
        broadcast: true
      }
    };
    return await this.interaction.sendData([message]);
  }
  static async pipe(connection, message) {
    return await connection.$__pipe(message);
  }
  /**
   * @description Передача сообщения из пространства имён в соединение
   */
  async $__pipe(chunk) {
    const event = chunk.meta.event;
    const payload = chunk.payload;
    const events = this.events.get(event);
    if (!events) return [];
    this.events.set(
      event,
      events.filter((event2) => event2.type === 0 /* ON */)
    );
    const results = await Promise.allSettled(
      events.map(async (eventInstance) => {
        const data = await eventInstance.listener(
          payload.data,
          getContentTypeString(payload["Content-Type"]),
          getContentEncodingString(payload["Content-Encoding"])
        );
        return this.createAck(chunk.meta.uuid, data, eventInstance.contentType, eventInstance.contentEncoding);
      })
    );
    return results.reduce((acc, cur) => {
      if (cur.status === "fulfilled" && cur.value) acc.push(cur.value);
      return acc;
    }, []);
  }
  //#endregion
  //#region Вспомогательные методы
  /**
   * @description Добавление слушателя события в карту событий
   */
  addEventListener(event, listener, type, contentType, contentEncoding) {
    if (!this.events.has(event)) {
      this.events.set(event, []);
    }
    this.events.get(event).push({
      type,
      listener,
      contentType,
      contentEncoding
    });
  }
  /**
   * @description Создание нового сообщения в формате IQSocketProtocolChunk
   */
  createAck(uuid, data, contentType, contentEncoding) {
    return {
      payload: {
        data,
        "Content-Type": determineContentType(data, contentType),
        "Content-Encoding": determineContentEncoding(contentEncoding)
      },
      meta: {
        type: protocol.EQSocketProtocolMessageType.ACK,
        uuid
      }
    };
  }
  //#endregion
};

// src/core/QSocketNamespace.ts
var QSocketNamespace = class {
  //#endregion
  //#region Конструктор
  /**
   * @constructor
   * @param {string} name - Название нового пространства имён.
   * @throws {Error} Если пространство имён с таким именем уже существует.
   * @description Создаёт новое пространство имён и добавляет его имя в список уникальных.
   */
  constructor(name, debuger) {
    /**
     * @private
     * @type {((connection: QSocketConnection) => void)[]}
     * @description Список слушателей для событий подключения клиента к пространству имён.
     */
    this.connectionListeners = [];
    /**
     * @private
     * @type {Map<QSocketInteraction, QSocketConnection>}
     * @description Карта активных соединений, связывающая клиентов с их подключениями.
     */
    this.connections = /* @__PURE__ */ new Map();
    this._name = name;
    this.debuger = debuger;
  }
  //#endregion
  //#region Свойства
  /**
   * @public
   * @type {string}
   * @description Возвращает название пространства имён.
   */
  get name() {
    return this._name;
  }
  //#endregion
  //#region Методы событий
  /**
   * @public
   * @param {'connection'} event - Событие, которое требуется отслеживать. В данный момент поддерживается только 'connection'.
   * @param {(connection: QSocketConnection) => void} listener - Слушатель, который будет вызван при наступлении события.
   * @throws {Error} Если событие не поддерживается.
   * @description Добавляет слушателя на событие подключения нового клиента к пространству имён.
   */
  on(event, listener) {
    switch (event) {
      case "connection":
        this.connectionListeners.push(listener);
        break;
      default:
        throw new Error(`\u041D\u0435\u0438\u0437\u0432\u0435\u0441\u0442\u043D\u043E\u0435 \u0441\u043E\u0431\u044B\u0442\u0438\u0435: ${event}`);
    }
  }
  //#endregion
  //#region Методы управления потоком данных
  /**
   * @public
   * @static
   * @param {QSocketNamespace} namespace - Экземпляр пространства имён, из которого будет удалён клиент.
   * @param {IQSocketProtocolChunk<IQSocketProtocolMessageMetaData>} message - Сообщение по протоколу Q-SOCKET-DATA.
   * @description Организует поток данных от клиента к активным соединениям передаваемого пространства имён.
   */
  static async pipe(interaction, namespace, message) {
    return await namespace.$__pipe(interaction, message);
  }
  /**
   * @private
   * @param {IQSocketProtocolChunk<IQSocketProtocolMessageMetaData>} message - Сообщение по протоколу Q-SOCKET-DATA.
   * @description Организует поток данных от клиента к активным соединениям текущего пространства имён.
   */
  async $__pipe(interaction, message) {
    const connection = this.connections.get(interaction);
    if (!connection) {
      return [];
    }
    return await QSocketConnection.pipe(connection, message);
  }
  //#endregion
  //#region Методы управления клиентами
  /**
   * @public
   * @static
   * @param {QSocketNamespace} namespace - Экземпляр пространства имён, к которому добавляется клиент.
   * @param {QSocketInteraction} interaction - Экземпляр клиента, который будет добавлен.
   * @description Добавляет клиента к указанному пространству имён, вызывая метод экземпляра для подключения.
   */
  static addClient(namespace, interaction) {
    namespace.$__addClient(interaction);
  }
  /**
   * @private
   * @param {QSocketInteraction} interaction - Экземпляр клиента для добавления.
   * @description Создаёт соединение для клиента, добавляет его в карту подключений и вызывает всех слушателей события "connection".
   */
  $__addClient(interaction) {
    const connection = new QSocketConnection(interaction, this);
    this.connections.set(interaction, connection);
    this.connectionListeners.forEach((listener) => {
      try {
        listener(connection);
      } catch (error) {
        this.debuger.error("Connection event error:", error);
      }
    });
  }
  /**
   * @public
   * @static
   * @param {QSocketNamespace} namespace - Экземпляр пространства имён, из которого будет удалён клиент.
   * @param {QSocketInteraction} interaction - Экземпляр клиента, который будет удалён.
   * @description Удаляет клиента из указанного пространства имён, вызывая метод экземпляра для отключения.
   */
  static deleteClient(namespace, interaction) {
    namespace.$__deleteClient(interaction);
  }
  /**
   * @private
   * @param {QSocketInteraction} interaction - Экземпляр клиента для удаления.
   * @description Удаляет соединение клиента из карты подключений и закрывает его.
   */
  $__deleteClient(interaction) {
    const connection = this.connections.get(interaction);
    this.connections.delete(interaction);
    if (connection !== void 0) {
      QSocketConnection.close(connection);
    }
  }
  //#endregion
  /**
   * @public
   * @description Уничтожает текущее пространство имён: удаляет все активные соединения клиентов и очищает его из списка уникальных имён.
   */
  destroy() {
    this.connections.forEach((_, interaction) => {
      this.$__deleteClient(interaction);
    });
  }
};

// src/core/QSocketDebuger.ts
var QSocketDebuger = class {
  /**
   * Конструктор класса QSocketUtils.
   * @param {IQSocketDebugConfig} [debugConfig] - Конфигурация для режима отладки.
   */
  constructor(debugConfig) {
    /** Логгер, используемый для вывода сообщений */
    this.logger = console;
    var _a;
    this.debug = (_a = debugConfig == null ? void 0 : debugConfig.on) != null ? _a : false;
    this.logger = (debugConfig == null ? void 0 : debugConfig.logger) || console;
    this.prefix = (debugConfig == null ? void 0 : debugConfig.prefix) || "[Q-SOCKET]";
  }
  /**
   * Логирует сообщение, если включен режим отладки.
   * @param {...any[]} message - Сообщение или данные для логирования.
   */
  log(...message) {
    if (this.debug) this.logger.log(this.prefix, ...message);
  }
  /**
   * Логирует сообщение об ошибке, если включен режим отладки.
   * @param {...any[]} message - Сообщение или данные для логирования ошибок.
   */
  error(...message) {
    if (this.debug) this.logger.error(this.prefix, ...message);
  }
  /**
   * Логирует информационное сообщение, если включен режим отладки.
   * @param {...any[]} message - Сообщение или данные для информационного логирования.
   */
  info(...message) {
    if (this.debug) this.logger.info(this.prefix, ...message);
  }
  /**
   * Логирует предупреждение, если включен режим отладки.
   * @param {...any[]} message - Сообщение или данные для логирования предупреждений.
   */
  warn(...message) {
    if (this.debug) this.logger.warn(this.prefix, ...message);
  }
  //#endregion
};

// src/core/QSocketUniqueGenerator.ts
var MAX_VALUE = Number.MAX_SAFE_INTEGER;
var QSocketUniqueGenerator = class {
  constructor(prefix = "") {
    /**
     * Текущий индекс для генерации UUID.
     * @private
     */
    this.uuidIndex = 0;
    this.prefix = prefix;
  }
  /**
   * Метод для генерации следующего уникального идентификатора.
   */
  next() {
    if (++this.uuidIndex > MAX_VALUE) this.uuidIndex = 0;
    return `${this.prefix}${this.uuidIndex.toString(16)}`;
  }
};

// src/core/QSocketInteraction.ts
var QSocketInteraction = class {
  constructor(id, socket, server, interactions, protocolConfig, debuger) {
    this.acks = /* @__PURE__ */ new Map();
    this.namespaces = /* @__PURE__ */ new Map();
    this.id = id;
    this.uuid = new QSocketUniqueGenerator(`${this.id}-M`);
    this.debuger = debuger;
    this.socket = socket;
    this.server = server;
    this.interactions = interactions;
    this.protocol = this.protocolConfigure(protocolConfig);
    this.socket.on("message", this.onHandle.bind(this));
  }
  static addNamespace(interaction, namespace) {
    interaction.namespaces.set(namespace.name, namespace);
    QSocketNamespace.addClient(namespace, interaction);
  }
  protocolConfigure(protocolConfig) {
    var _a, _b, _c;
    if ((_a = protocolConfig.compressor) == null ? void 0 : _a.on) {
      const compressor = (_b = protocolConfig.compressor.compressor) != null ? _b : new QSocketCompressor();
      const compressionFromSize = (_c = protocolConfig.compressor.compressionFromSize) != null ? _c : 100 * 1024;
      return new protocol.QSocketProtocol(compressor, compressionFromSize);
    } else {
      return new protocol.QSocketProtocol();
    }
  }
  /** Главный обработчик сообщений */
  async onHandle(data) {
    if (!Buffer.isBuffer(data)) {
      this.debuger.error("\u041E\u0431\u0449\u0435\u043D\u0438\u0435 \u043F\u043E \u043F\u0440\u043E\u0442\u043E\u043A\u043E\u043B\u0443 QSocket \u0432\u043E\u0437\u043C\u043E\u0436\u043D\u043E \u0442\u043E\u043B\u044C\u043A\u043E \u0432 \u0444\u043E\u0440\u043C\u0430\u0442\u0435 \u0431\u0443\u0444\u0435\u0440\u0430");
      return;
    }
    const message = await this.protocol.from(data);
    if (message instanceof protocol.QSocketProtocolDecodeError) {
      this.debuger.error(message);
      return;
    }
    const ackChunks = [];
    message.forEach((chunk) => {
      switch (chunk.meta.type) {
        case protocol.EQSocketProtocolMessageType.DATA:
          this.onData(chunk);
          break;
        case protocol.EQSocketProtocolMessageType.ACK:
          ackChunks.push(chunk);
          break;
        case protocol.EQSocketProtocolMessageType.CONTROL:
          this.onControl(chunk);
          break;
      }
    });
    if (ackChunks.length > 0) {
      this.onAck(ackChunks);
    }
  }
  onData(message) {
    const namespaceInstance = this.namespaces.get(message.meta.namespace);
    if (!namespaceInstance) {
      this.debuger.error(`Namespace "${message.meta.namespace}" does not exist`);
      return;
    }
    QSocketNamespace.pipe(this, namespaceInstance, message).then((results) => {
      this.sendAck(results);
    }).catch((error) => {
      this.debuger.error(`\u041E\u0448\u0438\u0431\u043A\u0430 \u043F\u0440\u0438 \u043E\u0431\u0440\u0430\u0431\u043E\u0442\u043A\u0435 \u0434\u0430\u043D\u043D\u044B\u0445: ${error.message}`);
      this.sendAck([
        {
          meta: {
            type: protocol.EQSocketProtocolMessageType.ACK,
            uuid: message.meta.uuid
          },
          payload: {
            data: `Error: ${error.message}`,
            "Content-Type": protocol.EQSocketProtocolContentType.STRING,
            "Content-Encoding": protocol.EQSocketProtocolContentEncoding.RAW
          }
        }
      ]);
    });
  }
  onAck(message) {
    const ackMap = /* @__PURE__ */ new Map();
    message.forEach((chunk) => {
      let ack = ackMap.get(chunk.meta.uuid) || [];
      ack.push(chunk);
      ackMap.set(chunk.meta.uuid, ack);
    });
    ackMap.forEach((value, uuid) => {
      const resolve = this.acks.get(uuid);
      if (!resolve) {
        this.debuger.error(`Return message UUID not found [id: ${this.id}, uuid: ${uuid}]`);
        return;
      }
      resolve(value.map((item) => item.payload));
      this.acks.delete(uuid);
    });
  }
  onControl(message) {
    let result = false;
    const data = message.payload.data;
    if (data.command === "join-namespace") {
      const namespace = this.server.getNamespace(data.namespace);
      if (!namespace) {
        this.debuger.error(`Namespace "${data.namespace}" not found`);
        return;
      }
      if (!this.namespaces.has(namespace.name)) {
        this.namespaces.set(namespace.name, namespace);
      }
      QSocketNamespace.addClient(namespace, this);
      result = true;
    } else if (data.command === "leave-from-namespace") {
      if (typeof data.namespace === "string") {
        const namespace = this.namespaces.get(data.namespace);
        if (!namespace) {
          this.debuger.error(`Namespace "${data.namespace}" not found`);
          return;
        }
        this.namespaces.delete(namespace.name);
        QSocketNamespace.deleteClient(namespace, this);
        result = true;
      }
    } else {
      this.debuger.error("Unknown control command");
    }
    this.sendAck([
      {
        meta: {
          type: protocol.EQSocketProtocolMessageType.ACK,
          uuid: message.meta.uuid
        },
        payload: {
          data: result,
          "Content-Type": protocol.EQSocketProtocolContentType.BOOLEAN,
          "Content-Encoding": protocol.EQSocketProtocolContentEncoding.RAW
        }
      }
    ]);
  }
  async sendData(message) {
    const data = await this.protocol.to(message);
    if (data instanceof protocol.QSocketProtocolEncodeError) {
      this.debuger.error(data);
      return;
    }
    return await this.sendSourceData(message, data);
  }
  async broadcast(message) {
    const promises = [];
    this.interactions.forEach((interaction) => {
      if (interaction !== this) {
        promises.push(interaction.sendData(message));
      }
    });
    return await Promise.allSettled(promises).then((result) => result.filter((item) => item.status === "fulfilled").map(({ value }) => value));
  }
  async sendSourceData(message, data) {
    this.socket.send(data);
    return (await Promise.allSettled(
      message.map((chunk) => {
        return new Promise((emitResolve, emitReject) => {
          const timeout = 1e4;
          const ackResolver = (ackResult) => {
            clearTimeout(timer);
            this.acks.delete(chunk.meta.uuid);
            emitResolve(ackResult);
          };
          this.acks.set(chunk.meta.uuid, ackResolver);
          const timer = setTimeout(() => {
            this.acks.delete(chunk.meta.uuid);
            this.debuger.error(`\u0412\u0440\u0435\u043C\u044F \u043E\u0436\u0438\u0434\u0430\u043D\u0438\u044F \u0438\u0441\u0442\u0435\u043A\u043B\u043E [event: ${chunk.meta.event}, uuid: ${chunk.meta.uuid}]`);
            emitReject(new Error("Timeout"));
          }, timeout);
        });
      })
    )).filter((res) => res.status === "fulfilled").map(({ value }) => value);
  }
  async sendAck(message) {
    const data = await this.protocol.to(message);
    this.socket.send(data);
  }
  async sendCommand(message) {
    const data = await this.protocol.to(message);
    if (data instanceof protocol.QSocketProtocolEncodeError) {
      this.debuger.error(data);
      return;
    }
    this.socket.send(data);
    return (await Promise.allSettled(
      message.map((chunk) => {
        return new Promise((emitResolve, emitReject) => {
          const timeout = 1e4;
          const ackResolver = (ackResult) => {
            clearTimeout(timer);
            emitResolve(ackResult);
            this.acks.delete(chunk.meta.uuid);
          };
          this.acks.set(chunk.meta.uuid, ackResolver);
          const timer = setTimeout(() => {
            this.debuger.error(`\u0412\u0440\u0435\u043C\u044F \u043E\u0436\u0438\u0434\u0430\u043D\u0438\u044F \u0438\u0441\u0442\u0435\u043A\u043B\u043E [command, uuid: ${chunk.meta.uuid}]`);
            emitReject(new Error("Timeout"));
            this.acks.delete(chunk.meta.uuid);
          }, timeout);
        });
      })
    )).filter((res) => res.status === "fulfilled").map(({ value }) => value);
  }
};

// src/interfaces/QSocketBase.ts
var clientUUID = new QSocketUniqueGenerator();
var serverUUID = new QSocketUniqueGenerator();
var QSocketBase = class {
  /**
   * Конструктор класса QSocketBase.
   * @param {IQSocketDebugConfig} [debugConfig] - Конфигурация для режима отладки.
   */
  constructor(type, protocolConfig, debugConfig) {
    this.namespaces = /* @__PURE__ */ new Map();
    this.interactions = /* @__PURE__ */ new Map();
    this.protocolConfig = getDefaultProtocolConfig();
    var _a, _b, _c;
    this.type = type;
    this.debuger = new QSocketDebuger(debugConfig);
    if (protocolConfig !== void 0 && protocolConfig.compressor !== void 0) {
      this.protocolConfig.compressor.on = (_a = protocolConfig.compressor.on) != null ? _a : this.protocolConfig.compressor.on;
      this.protocolConfig.compressor.compressor = (_b = protocolConfig.compressor.compressor) != null ? _b : this.protocolConfig.compressor.compressor;
      this.protocolConfig.compressor.compressionFromSize = (_c = protocolConfig.compressor.compressionFromSize) != null ? _c : this.protocolConfig.compressor.compressionFromSize;
    }
    const prefix = type === "server" ? "S" : "C";
    const generator = type === "server" ? serverUUID : clientUUID;
    this.id = `${prefix}${generator.next()}`;
    this.uuid = new QSocketUniqueGenerator(`${this.id}-SM`);
    this.interactionUUID = new QSocketUniqueGenerator(`${this.id}-I`);
  }
  connectionHandle(socket, server) {
    const interactionId = this.interactionUUID.next();
    const interaction = new QSocketInteraction(interactionId, socket, server, this.interactions, this.protocolConfig, this.debuger);
    this.interactions.set(interactionId, interaction);
    socket.on("close", () => {
      this.interactions.delete(interactionId);
    });
  }
  /**
   * Создаёт новое пространство имён или возвращает существующее.
   * @param {string} name - Имя создаваемого пространства имён.
   * @returns {QSocketNamespace} Пространство имён QSocket.
   */
  createNamespace(name) {
    if (this.namespaces.has(name)) {
      this.debuger.warn(`\u041F\u0440\u043E\u0441\u0442\u0440\u0430\u043D\u0441\u0442\u0432\u043E \u0438\u043C\u0451\u043D '${name}' \u0443\u0436\u0435 \u0441\u0443\u0449\u0435\u0441\u0442\u0432\u0443\u0435\u0442`);
      return this.namespaces.get(name);
    }
    const namespace = new QSocketNamespace(name, this.debuger);
    this.namespaces.set(name, namespace);
    this.debuger.info(`\u041F\u0440\u043E\u0441\u0442\u0440\u0430\u043D\u0441\u0442\u0432\u043E \u0438\u043C\u0451\u043D "${name}" \u0441\u043E\u0437\u0434\u0430\u043D\u043E!`);
    this.namespaceHandle(namespace);
    return namespace;
  }
  /**
   * Удаляет существующее пространство имён.
   * @param {string} name - Имя удаляемого пространства имён.
   * @returns {boolean} Возвращает `true`, если пространство имён было удалено, иначе `false`.
   */
  removeNamespace(name) {
    if (!this.namespaces.has(name)) {
      this.debuger.warn(`\u041F\u0440\u043E\u0441\u0442\u0440\u0430\u043D\u0441\u0442\u0432\u043E \u0438\u043C\u0451\u043D '${name}' \u043D\u0435 \u0441\u0443\u0449\u0435\u0441\u0442\u0432\u0443\u0435\u0442`);
      return false;
    }
    this.namespaces.get(name).destroy();
    this.namespaces.delete(name);
    this.debuger.info(`\u041F\u0440\u043E\u0441\u0442\u0440\u0430\u043D\u0441\u0442\u0432\u043E \u0438\u043C\u0451\u043D "${name}" \u0443\u043D\u0438\u0447\u0442\u043E\u0436\u0435\u043D\u043E!`);
    return true;
  }
  /**
   * Получает пространство имён по его имени.
   * @param {string} name - Имя запрашиваемого пространства имён.
   * @returns {QSocketNamespace | undefined} Пространство имён или `undefined`, если оно не найдено.
   */
  getNamespace(name) {
    return this.namespaces.get(name);
  }
};
var QSocketClient = class extends QSocketBase {
  constructor(socket, protocolConfig, debugConfig) {
    super("client", protocolConfig, debugConfig);
    this.connectionHandle(socket, this);
  }
  namespaceHandle(namespace) {
    const message = [
      {
        meta: {
          type: protocol.EQSocketProtocolMessageType.CONTROL,
          uuid: this.uuid.next(),
          namespace: namespace.name
        },
        payload: {
          data: {
            command: "join-namespace",
            namespace: namespace.name
          },
          "Content-Type": protocol.EQSocketProtocolContentType.JSON,
          "Content-Encoding": protocol.EQSocketProtocolContentEncoding.RAW
        }
      }
    ];
    this.interactions.forEach((interaction) => {
      interaction.sendCommand(message).then(() => QSocketInteraction.addNamespace(interaction, namespace)).catch(() => this.debuger.error(`\u041E\u0448\u0438\u0431\u043A\u0430 \u043F\u0440\u0438 \u043F\u043E\u0434\u043A\u043B\u044E\u0447\u0435\u043D\u0438\u0438 \u043A \u043F\u0440\u043E\u0441\u0442\u0440\u0430\u043D\u0441\u0442\u0432\u0443 "${namespace.name}"`));
    });
  }
};

// src/interfaces/QSocketServer.ts
var QSocketServer = class extends QSocketBase {
  constructor(transport, protocolConfig, debugConfig) {
    super("server", protocolConfig, debugConfig);
    this.server = transport;
    this.server.on("connection", (socket) => this.connectionHandle(socket, this));
  }
  namespaceHandle() {
  }
};

exports.QSocketClient = QSocketClient;
exports.QSocketServer = QSocketServer;
//# sourceMappingURL=index.cjs.map
//# sourceMappingURL=index.cjs.map